@@ -14,6 +14,13 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+enable_logging = False
+
+if enable_logging:
+    fd = open('/tmp/openvpn-applet.log','w+')
+else:
+    fd = None
+
 import glob
 import os
 import string
@@ -93,9 +100,9 @@ class vpn():
                 
         if pid != None and pid_exists(pid) and openvpn_running(pid):
             #self.pid = pid
-            return 0
+            return True
         else:
-            return -1
+            return False
         
     def get_pid(self):
         print "pid_file: ",self.pid_file
@@ -108,10 +115,10 @@ class vpn():
         return int(pid)
         
     def start(self,user=None,passwd=None,enable_log=False):
-        print "start",user,passwd
         command =["sudo",self.startcmd,"--daemon","--writepid",self.pid_file,"--config",self.conf_file]
         if enable_log:
-            command.extend(['--log',self.log_file])
+            command.extend(['--log',self.log_file,'--verb','3'])
+        print command
         process = Popen(command, shell=False, stdin=PIPE, stdout=None, cwd=self.conf_dir)
         input = process.stdin
               
@@ -127,7 +134,6 @@ class vpn():
         
     def stop(self):
         print "stop"
-        #status = call(['sudo', self.cmd, 'stop', self.name, self.conf_dir])
         status = call(['sudo', self.stopcmd, 'stop', self.name])
         return status
 
@@ -147,8 +153,8 @@ class vpn():
         return self.log_file
 
     def __read_conf(self):
-        parameters = {}
         keywords = ["askpass","auth-user-pass"]
+        parameters = dict.fromkeys(keywords,False)
         
         line = None
         for line in fileinput.input(self.conf_file):
@@ -159,7 +165,7 @@ class vpn():
                 for word in keywords:
                     #print word
                     if word == s[0]:
-                        print "MATCH: ", word
+                        print "MATCH: ", word,s
                         #get filename
                         if len(s) > 1:
                             filename = line.split()[1]
@@ -169,15 +175,9 @@ class vpn():
                             elif os.path.exists(os.path.join(self.conf_dir, filename)):
                                 parameters[word] = filename
                             else:
-                                parameters[word] = None
+                                parameters[word] = True
                         else:
                             parameters[word] = True
-                    else:
-                        parameters[word] = False
-
-        if line == None:
-            for word in keywords:
-                parameters[word] = False
 
         return parameters
     
@@ -191,17 +191,17 @@ class vpn_control():
         
     def __init__(self):
         self.vpn_list = []
-        for v in self.read_vpn_conf('/etc/openvpn',['conf']):
-            print v
+        for v in self.read_vpn_conf('/etc/openvpn',['conf','ovpn']):
+            log(v)
             self.vpn_list.append(vpn(v))
-        for v in self.read_vpn_conf('/home/user/.openvpn-applet',['conf']):
-            print v
+        for v in self.read_vpn_conf('/home/user/.openvpn-applet',['conf','ovpn']):
+            log(v)
             self.vpn_list.append(vpn(v,'/home/user/.openvpn-applet'))
     
     def __str__(self):
         s = ''
         for i,v in enumerate(self.vpn_list):
-            if v.get_status() != 0: cmd = 'start'
+            if v.get_status() != True: cmd = 'start'
             else: cmd = 'stop'
             s = s+str(i+1)+'. '+v.name+' - '+cmd+'\n'
         return s   
@@ -226,6 +226,9 @@ class vpn_control():
         
     def pass_required(self,index):
         return self.vpn_list[index].pass_required()
+        
+    def get_vpn_count(self):
+        return len(self.vpn_list)
 
 
     
@@ -258,10 +261,14 @@ class openvpn_applet(hildondesktop.HomeItem):
         self.button.connect("clicked", self.button_clicked_event)
         self.hbox.pack_start(self.button)
         
-        cell = gtk.CellRendererText()
+        textcell = gtk.CellRendererText()
+        imagecell = gtk.CellRendererPixbuf()
+        
         self.combo = gtk.ComboBox()
-        self.combo.pack_start(cell, True)
-        self.combo.add_attribute(cell, 'text', 0)
+        self.combo.pack_start(imagecell, True)
+        self.combo.pack_start(textcell, True)
+        self.combo.add_attribute(imagecell, 'stock-id',3)
+        self.combo.add_attribute(textcell, 'text', 0)
 
         self.vseparator = gtk.VSeparator()
         self.hbox.pack_start(self.vseparator)
@@ -278,16 +285,47 @@ class openvpn_applet(hildondesktop.HomeItem):
         self.update_list()
         self.show_all()
     
-    def update_list(self):
-        vpn_list = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_BOOLEAN)
+    def update_list(self,source=None):
+        vpn_list = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING,gobject.TYPE_PYOBJECT,gobject.TYPE_STRING,gobject.TYPE_BOOLEAN)
         self.vc = vpn_control()
-        for i,v in enumerate(self.vc.vpn_list):
-            vpn_list.insert(i,(v.get_name(), v.get_conf_file(),True))
-            #self.combo.insert_text(i,v.name)
+        if len(self.vc.vpn_list) == 0:
+            #vpn_list.insert(0,("No Connections", "",None,None))
+            pass
+        else:
+            for i,v in enumerate(self.vc.vpn_list):
+                if v.get_status():
+                    icon = gtk.STOCK_YES
+                else:
+                    icon = gtk.STOCK_NO
+                vpn_list.insert(i,(v.get_name(), v.get_conf_file(),v,icon,True))
+                #self.combo.insert_text(i,v.name)
+
         self.combo.set_model(vpn_list)
-        if len(self.vc.vpn_list) > 0:
+        if source == None:
+            self.set_options_visible(True)
+            self.combo.set_active(0)
+
+        
+        if self.vc.get_vpn_count() > 0:
             self.combo.set_active(0)
         self.frame.check_resize()
+
+    def set_options_visible(self,available):
+        model = self.combo.get_model()
+        iter = model.get_iter_first()
+        if iter == None:
+            if available == True:
+                model.append(("Options...",None,None,None,False))
+        else:    
+            while model.iter_next(iter) != None:
+                iter = model.iter_next(iter)
+    
+            if available == True:
+                if model.get_value(iter,4) == True:
+                    model.append(("Options...",None,None,None,False))
+            else:
+                if model.get_value(iter,4) == False:
+                    model.remove(iter)            
     
     def get_vpn_list(self):
         return self.combo.get_model()
@@ -295,15 +333,17 @@ class openvpn_applet(hildondesktop.HomeItem):
     def combo_changed_event(self,widget):
         index = self.combo.get_active()
         if index >= 0:
-            if self.vc.status(index) != 0:
+            if index == self.vc.get_vpn_count():
+                self.button.set_label("Open")
+            elif self.vc.status(index) == False:
                 self.button.set_label("Start")
             else:
                 self.button.set_label("Stop")
     
     def button_clicked_event(self,widget):
         index = self.combo.get_active()
-        if index >= 0:
-            if self.vc.status(index) != 0:
+        if index >= 0  and index != self.vc.get_vpn_count():
+            if self.vc.status(index) == False:
                 if self.vc.user_required(index) and self.vc.pass_required(index):
                     (user,passwd,cont) = self.user_pass_dlg(True)
                 elif self.vc.pass_required(index):
@@ -318,9 +358,11 @@ class openvpn_applet(hildondesktop.HomeItem):
             else:
                 self.banner = hildon.hildon_banner_show_information(self,None,"Stopping Openvpn Connection")
                 self.vc.stop(index)
-        self.timeout = 50
-        self.timer = gobject.timeout_add(200, self.update_button)
-
+            self.timeout = 50
+            self.timer = gobject.timeout_add(200, self.update_button)
+        elif index == self.vc.get_vpn_count():
+            print "open settings"
+            self.open_settings(self)
     
     def user_pass_dlg(self,ask_passwd):
         if ask_passwd:
@@ -346,18 +388,26 @@ class openvpn_applet(hildondesktop.HomeItem):
     def update_button(self):
         self.timeout = self.timeout - 1
         index = self.combo.get_active()
+        if index == self.vc.get_vpn_count():
+            self.timeout = 0
+            self.timer = None
+            return False
+            
         if self.timeout > 0:
-            if self.vc.status(index) != 0:
+            if self.vc.status(index) == False:
                 new_label = "Start"
             else:
                 new_label = "Stop"
             
             if self.button.get_label() != new_label:
-                self.button.set_label(new_label)
+                self.update_list()
+                self.combo.set_active(index)
                 self.timeout = 0
                 self.timer = None
             else:
                 self.timer = gobject.timeout_add(200, self.update_button)
+        else:
+            return False
 
     def settings_event(self, widget, data=None):
         widget = gtk.MenuItem("Openvpn setup")
@@ -365,19 +415,23 @@ class openvpn_applet(hildondesktop.HomeItem):
         return widget
         
     def open_settings(self, widget, data=None):
+        self.set_options_visible(False)
         dlg = OpenvpnAppletSettings(model=self.combo.get_model(), applet=self)
         dlg.run()
         dlg.destroy()
-        
+        self.set_options_visible(True)
+        self.combo.set_active(0)
 
 class OpenvpnAppletSettings(gtk.Dialog):
     def __init__(self,title="Openvpn connections", parent=None, flags=0, buttons=None, model=None, applet=None):
         gtk.Dialog.__init__(self,title, parent, flags, buttons)
+
+        self.model = model
+
         
-        self.mode = model
         self.applet = applet
         
-        self.treeview = gtk.TreeView(model)
+        self.treeview = gtk.TreeView(self.model)
         self.vpncolumn = gtk.TreeViewColumn('Openvpn Connection')
         self.treeview.append_column(self.vpncolumn)
         self.cell = gtk.CellRendererText()
@@ -390,8 +444,8 @@ class OpenvpnAppletSettings(gtk.Dialog):
         self.delete_button.connect("clicked", self.delete_clicked_event)
         #self.disable_button = gtk.Button("Disable")
         #self.disable_button.connect("clicked", self.disable_clicked_event)
-        #self.test_button = gtk.Button("Disable")
-        #self.test_button.connect("clicked", self.test_clicked_event)
+        self.test_button = gtk.Button("Test")
+        self.test_button.connect("clicked", self.test_clicked_event)
 
 
         self.treeview.set_size_request(-1, 155)
@@ -403,6 +457,7 @@ class OpenvpnAppletSettings(gtk.Dialog):
         self.vbox.pack_start(hbox)   
         self.action_area.pack_start(self.new_button)
         self.action_area.pack_start(self.delete_button)
+        self.action_area.pack_start(self.test_button)
         self.add_button("Close", gtk.RESPONSE_CLOSE)
         self.action_area.show_all()
         self.vbox.show_all()
@@ -412,22 +467,47 @@ class OpenvpnAppletSettings(gtk.Dialog):
         dlg = OpenvpnAppletNewConnection()
         response = dlg.run()
         if response == gtk.RESPONSE_OK:
-            copy_file(dlg.get_filenames())
-            self.applet.update_list()
+            files_to_overwrite = {}
+            files_to_overwrite_message = []
+            files_to_copy = {}
+            all_files = dlg.get_filenames()
+            for f in all_files:
+                target_file = os.path.join('/etc/openvpn', os.path.basename(f))
+                if os.path.isfile(target_file):
+                    files_to_overwrite[f] = all_files[f]
+                    files_to_overwrite_message.append(target_file)
+                else:
+                    files_to_copy[f] =  all_files[f]
+            if len(files_to_overwrite) > 0:
+                print "Files to overwrite:", files_to_overwrite
+                mstr = ""
+                for f in files_to_overwrite_message:
+                    mstr = mstr+"\n"+f
+                mdlg = gtk.MessageDialog(dlg,0,gtk.MESSAGE_QUESTION,gtk.BUTTONS_NONE,"Overwrite files?\n"+mstr)
+                mdlg.add_buttons("Overwrite", gtk.RESPONSE_ACCEPT,"Don't Overwrite", gtk.RESPONSE_REJECT)
+                response = mdlg.run()
+                mdlg.destroy()
+                if response == gtk.RESPONSE_ACCEPT:
+                    files_to_copy = all_files
+            else:
+                files_to_copy = all_files
+                        
+            copy_file(files_to_copy)
+            self.applet.update_list(self)
             self.treeview.set_model(self.applet.get_vpn_list())
         dlg.destroy()
 
     def delete_clicked_event(self, widget, data=None):
-        print "delete_clicked_event"
+        log("delete_clicked_event")
         file_links = FileLinks("/etc/openvpn", ["conf", "ovpn"])
         selection = self.treeview.get_selection()
         selection.set_mode(gtk.SELECTION_SINGLE)
         (model, iter) = selection.get_selected()
         if iter != None:
-            print "Item selected"
+            log("Item selected")
             filename = model.get_value(iter,1)
-            print filename
-            print file_links[filename]
+            log(filename)
+            log(file_links[filename])
             mstr = filename
             for f in file_links[filename]:
                 mstr += "\n"+str(f)
@@ -439,85 +519,127 @@ class OpenvpnAppletSettings(gtk.Dialog):
                 delete_files(file_links[filename])
                 delete_files([filename])
         else:
-            print "No selection"
+            log("No selection")
 
-        self.applet.update_list()
+        self.applet.update_list(self)
         self.treeview.set_model(self.applet.get_vpn_list())
         
     def disable_clicked_event(self, widget, data=None):
         pass
     
-#    def test_clicked_event(self,widget,data=None):
-#        if self.applet.vc.user_required(index) and self.applet.vc.pass_required(index):
-#            (user,passwd,cont) = self.applet.user_pass_dlg(True)
-#        elif self.vc.pass_required(index):
-#            (user,passwd,cont) = self.applet.user_pass_dlg(False)
-#        else:
-#            user   = None
-#            passwd = None
-#            cont = True
-#        if cont:
-#            self.vc.start(index,user,passwd,True)
-#
-#        log_file_fd = os.open(vpn.get_log_file(),os.O_RDONLY)
-#        dlg = OpenvpnAppletTestConnection(log_file_fd)
-#        dlg.run()        
-#        dlg.destroy()
-#
-#        log_file_fd.close()
+    def test_clicked_event(self,widget,data=None):
+        selection = self.treeview.get_selection()
+        selection.set_mode(gtk.SELECTION_SINGLE)
+        (model, iter) = selection.get_selected()
+        if iter != None:
+            log("Item selected")
+            vpn = model.get_value(iter,2)
+            if vpn.user_required() and vpn.pass_required():
+                (user,passwd,cont) = self.applet.user_pass_dlg(True)
+            elif vpn.pass_required():
+                (user,passwd,cont) = self.applet.user_pass_dlg(False)
+            else:
+                user   = None
+                passwd = None
+                cont = True
+            if cont:
+                fd = open(vpn.get_log_file(),'w+')
+                fd.close()
+                if vpn.get_status:
+                    vpn.stop()
+                vpn.start(user,passwd,True)
 
-#class OpenvpnAppletTestConnection(gtk.Dialog):
-#    def __init__(self,title="Test Connection", parent=None, flags=0, buttons=None,log_file_fd=None):
-#        gtk.Dialog.__init__(self,title, parent, flags, buttons)
-#        
-#        self.fd = log_file_fd
-#        
-#        self.textbuffer = gtk.TextBuffer()
-#        self.textview = gtk.TextView(self.textbuffer) 
-#        
-#        scrolledwindow = gtk.ScrolledWindow()
-#        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
-#        scrolledwindow.add(self.textview)
-#        
-#        self.vbox.pack_start(scrolledwindow)
-#                
-#        button = gtk.Button(Close)
-#        self.add_button("Close",gtk.RESPONSE_CLOSE)
-#        
-#        self.update_text()
-#        self.show_all()
-#    
-#    def update_text(self):
-#        self.timer = gobject.timeout_add(1000, self.update_text)
-#       text = self.fd.read()
-#        self.textbuffer.set_text(text)
+            dlg = OpenvpnAppletTestConnection(parent=self,log_file=vpn.get_log_file())            
+            dlg.run()        
+            dlg.destroy()
+            vpn.stop()
+
+
+class OpenvpnAppletTestConnection(gtk.Dialog):
+    def __init__(self,title="Test Connection", parent=None, flags=0, buttons=None,log_file=None):
+        gtk.Dialog.__init__(self,title, parent, flags, buttons)
         
+        self.fd = open(log_file,"r+")
+        log(self.fd)
+        self.timer = None
+        
+        self.textbuffer = gtk.TextBuffer()
+        self.textview = gtk.TextView(self.textbuffer) 
+        self.textview.set_size_request(800, 480)
+        self.textview.set_editable(False)
+        self.textview.set_cursor_visible(False)
+        
+        scrolledwindow = gtk.ScrolledWindow()
+        scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC,gtk.POLICY_AUTOMATIC)
+        scrolledwindow.add(self.textview)
+        
+        self.vbox.pack_start(scrolledwindow)
+                
+        save_button = gtk.Button("Save log")
+        save_button.connect("clicked", self.save_button_clicked)
+        
+        self.action_area.pack_start(save_button)
+        self.add_button(gtk.STOCK_CLOSE,gtk.RESPONSE_CLOSE)
+        
+        self.update_text()
+        self.show_all()
+    
+    def update_text(self):
+        log("update_text")
+        if self.timer == 'stop':
+            return False
+        text = self.fd.read()
+        #TextIter.set_line forward_to_end
+        self.textbuffer.insert(self.textbuffer.get_end_iter(), text)
+        #self.textbuffer.insert_at_cursor(text)
+        self.timer = gobject.timeout_add(1000, self.update_text)
+    
+    def save_button_clicked(self, widget, data=None):
+        dlg = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_SAVE)
+        response = dlg.run()
+        if response == gtk.RESPONSE_OK:
+            filename = dlg.get_filename()
+            fd = open(filename,'w+')
+            print >>fd,self.textbuffer.get_text(self.textbuffer.get_start_iter(),self.textbuffer.get_end_iter())
+            fd.close()
+            
+        dlg.destroy()
+        
+    
+    def destroy(self):
+        self.timer = 'stop'
+        self.fd.close()
+        super(OpenvpnAppletTestConnection,self).destroy()
     
 class OpenvpnAppletNewConnection(gtk.Dialog):
     def __init__(self,title="New Connection", parent=None, flags=0, buttons=None):
         gtk.Dialog.__init__(self,title, parent, flags, buttons)
 
-        self.check = []
-        self.file_entry = []        
+        self.extra_files = []
+        self.check = {}
+        self.file_entry = {}        
 
         self.add_button("Import Selected",gtk.RESPONSE_OK)
         self.add_button("Cancel",gtk.RESPONSE_CANCEL)
-        #self.filechooser = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_OPEN)
-
-        #file_dialog = hildon.FileChooserDialog(self,gtk.FILE_CHOOSER_ACTION_OPEN)
-        descriptions = ["Configuration File:", "Key File:", "Cert File:", "Ca File:","Secret File:", "PKCS12 File:"]
+        description_keys = ["Configuration File:", "Key File:", "Cert File:", "Ca File:", "Secret File:", "PKCS12 File:"]
+        description_values = ["conf", "key", "cert", "ca", "secret", "pkcs12"]
+        descriptions = {}
+        for i,key in enumerate(description_keys):
+            descriptions[key] = description_values[i]
+        log(descriptions)
         table = gtk.Table(len(descriptions), 2)
-        for i,d in enumerate(descriptions):
+        for i,d in enumerate(description_keys):
+            log(d)
             hbox = gtk.HBox()
-            self.file_entry.append(gtk.Entry())
+            self.file_entry[descriptions[d]] = gtk.Entry()
             browse_button = gtk.Button("Browse")
 
-            hbox.pack_start(self.file_entry[i])
+            hbox.pack_start(self.file_entry[descriptions[d]])
             hbox.pack_start(browse_button)           
-            self.check.append(gtk.CheckButton(d))
-            browse_button.connect("clicked", self.browse_button_event, (self.file_entry[i], self.check[i]))   
+            self.check[descriptions[d]] = gtk.CheckButton(d)
+            browse_button.connect("clicked", self.browse_button_event, (self.file_entry[descriptions[d]], self.check[descriptions[d]]))   
 
-            table.attach(self.check[i],0,1,i,i+1)
+            table.attach(self.check[descriptions[d]],0,1,i,i+1)
             table.attach(hbox,1,2,i,i+1)   
             self.vbox.pack_start(table)
             
@@ -525,14 +647,18 @@ class OpenvpnAppletNewConnection(gtk.Dialog):
 
     def get_filenames(self):
         files = {}
-        for i,check in enumerate(self.check):
-            if check.get_active():
-                fname = self.file_entry[i].get_text()
+        for i,key in enumerate(self.check):
+            if self.check[key].get_active():
+                fname = self.file_entry[key].get_text()
                 perm = '600'
                 for ext in ["conf", "ovpn"]:
                     if re.search("\."+ext+"$",fname):
                         perm = '644'
-                files[self.file_entry[i].get_text()] = perm
+                files[self.file_entry[key].get_text()] = perm
+            
+            for file in self.extra_files:
+                files[file] = '755'
+            
         return files
     
     def browse_button_event(self, widget, data=None):
@@ -543,29 +669,62 @@ class OpenvpnAppletNewConnection(gtk.Dialog):
         if response == gtk.RESPONSE_OK:
             textentry.set_text(dlg.get_filename())
             checkbox.set_active(True)
+            filelinks = FileLinks(dlg.get_filename())
+            filelinks.filter(['/home','/media'])
+            print "Referenced files in",dlg.get_filename(),filelinks
+            for f in filelinks[dlg.get_filename()]:
+                print "Keyword",f,filelinks.get_matchword(f)
+                try:
+                    self.file_entry[filelinks.get_matchword(f)].set_text(f)
+                    self.check[filelinks.get_matchword(f)].set_active(True)
+                    print "Keyword file:",filelinks.get_matchword(f),f
+                except KeyError:
+                    self.extra_files.append(f)
+                    print "Extra file:",f
+                
+            if len(filelinks.get_not_found()) > 0:
+                print "Files not found:",filelinks.get_not_found()
+                mstr = ""
+                for f in filelinks.get_not_found():
+                    mstr = mstr+"\n"+f
+                mdlg = gtk.MessageDialog(self,0,gtk.MESSAGE_INFO,gtk.BUTTONS_OK,"Files not found:\n"+mstr)
+                response = mdlg.run()
+                mdlg.destroy() 
+            
         dlg.destroy()
         
 
 class FileLinks(UserDict):
-    def __init__(self, directory, filetypes):
+    def __init__(self, path, filetypes=None):
         UserDict.__init__(self)
-        self.directory = directory
+        self.path = path
         self.keywords = ["secret","up","down","ca","cert","key","pkcs12","iproute","status","dh","tls-verify","ipchange","client-connect","route-up","client-disconnect","learn-address","auth-user-pass-verify","askpass","auth-user-pass"]
         self.filetypes = filetypes
+        self.matchword = {}
+        self.files_not_found = []
+        if os.path.isdir(path):
+            self.type = 'directory'
+        else:
+            self.type = 'file'
+        
         self.update()
-    
+        
+        
     def update(self):
-        print "function: update"
+        log("function: update")
         self.data = {}
         self.references = {}
-        for root, dirs, files in os.walk(self.directory):
-            print root, files
-            for name in files:
-                print "name: ", name
-                for type in self.filetypes:
-                    if re.search(".*\."+type+"$",name):
-                        self.data[os.path.join(root, name)] = self.__scan(os.path.join(root, name), root)
-                
+        if self.type == 'directory':
+            for root, dirs, files in os.walk(self.path):
+                print root, files
+                for name in files:
+                    print "name: ", name
+                    for type in self.filetypes:
+                        if re.search(".*\."+type+"$",name):
+                            self.data[os.path.join(root, name)] = self.__scan(os.path.join(root, name), root)
+        else:
+            self.data[self.path] = self.__scan(self.path, os.path.dirname(self.path))
+        
     def __scan(self,file,root):
         print "function: __scan", file, root
         fileset = Set()
@@ -585,9 +744,12 @@ class FileLinks(UserDict):
                         if filename != None and os.path.exists(filename):
                             #add to return Set
                             fileset.add(filename)
+                            self.matchword[filename] = word
                         elif filename !=None and os.path.exists(os.path.join(root, filename)):
                             fileset.add(os.path.join(root, filename))
+                            self.matchword[os.path.join(root, filename)] = word
                         else:
+                            self.files_not_found.append(filename)
                             filename = None
                     
         #add entries in return Set to reference count
@@ -605,12 +767,40 @@ class FileLinks(UserDict):
             if self.references[it] == 1:
                 return_items.append(it)
         return return_items
-     
+
+    def get_matchword(self,file):
+        try:
+            response = self.matchword[file]     
+        except KeyError:
+            response = None
+        return response
+            
+    def get_not_found(self):
+        if self.files_not_found == [None]:
+            self.files_not_found = []
+        return self.files_not_found
         
+    def filter(self,whitelist):
+        for key in self.data:
+            passed_files = []
+            for file in self.data[key]:
+                if os.path.isabs(file):
+                    for wl in whitelist:
+                        if re.search('^'+wl,file):
+                            passed_files.append(file)
+                        else:
+                            print "Dropped:", file
+            self.data[key] = passed_files
+                        
+    
 def hd_plugin_get_objects():
 	plugin = openvpn_applet()
 	return [plugin]
 
+def log(input):
+    if enable_logging:
+        print input
+        print >>fd,input
 
 if __name__ == '__main__':
     vc = vpn_control()
